function absmean = estimate_abscoeff(room, Tr, estimator)
% ESTIMATE_ABSCOEFF - Average wall absorption coefficient from inverse form of Sabine's and Eyring's
% formulae.
%
% Usage:
%   absmean = ESTIMATE_ABSCOEFF(room, Tr, estimator)
%
% Input:
%   room        room structure (see RAZR)
%   Tr          Reverberation time in seconds
%   estimator   Underlying T60 estimator ('eyring' or 'sabine')
%
% Output:
%   absmean     Average wall absorption coefficient according to Tr values (row vector)

%------------------------------------------------------------------------------
% RAZR engine for Mathwork's MATLAB
%
% Version 0.96.3
%
% Author(s): Torben Wendt
%
% Copyright (c) 2014-2023, Torben Wendt, Steven van de Par, Stephan Ewert,
% University of Oldenburg, Germany.
%
% This work is licensed under the
% Creative Commons Attribution-NonCommercial-NoDerivs 4.0 International
% License (CC BY-NC-ND 4.0).
% To view a copy of this license, visit
% http://creativecommons.org/licenses/by-nc-nd/4.0/ or send a letter to
% Creative Commons, 444 Castro Street, Suite 900, Mountain View, California,
% 94041, USA.
%------------------------------------------------------------------------------


[Aeq, S] = eq_abs_surfarea(room);



const = 24*log(10)/speedOfSound(room);

switch estimator
    case 'sabine'
        absmean = const*prod(room.boxsize)./(S*Tr(:)');
    case 'eyring'
        absmean = 1 - exp(const*prod(room.boxsize)./(-S*Tr(:)'));
    otherwise
        error('Unknown estimator: %s', estimator);
end

% SE 2021 allow reflection coeff weight per wall to calculate abscoeffs
% I did not put this in
% eq_abs_surfarea because that function is also used to calculate the surf
% area at other places in the code
% FIXME has double code with eq_abs_surfarea
%room.absorption_wall_scale = [1 5.5 1 1 5.5 1];
if isfield(room, 'absorption_wall_scale') && ~isempty(room.absorption_wall_scale)
   % has to be specified as [-z -y -x x y z]
   k = [1 2; 3 1; 2 3; 3 2; 1 3; 2 1];  % (materials specified as [-z -y -x x y z])
   surfs = prod(room.boxsize(k), 2);
   %surfs = surfs.*room.absorption_wall_scale(:)/mean(room.absorption_wall_scale);
   
   %SE for each frequency
   if size(room.absorption_wall_scale, 1) == 6 && size(room.absorption_wall_scale, 2) == length(absmean)
       for idx = 1:length(absmean)
            surfs2 = surfs.*room.absorption_wall_scale(:,idx);
            S2 = sum(surfs2);
       
       absmean2(:,idx) = repmat(absmean(idx)*S/S2,6,1).*room.absorption_wall_scale(:,idx);
       end
   else
       % all the same
       surfs = surfs.*room.absorption_wall_scale(:);
       S2 = sum(surfs);
       
       absmean2 = absmean*S/S2;
       %absmean2 = repmat(absmean2,6,1).*repmat(room.absorption_wall_scale(:)/mean(room.absorption_wall_scale),1,length(absmean));
       absmean2 = repmat(absmean2,6,1).*repmat(room.absorption_wall_scale(:),1,length(absmean));
       
   end
   
   if any(any(absmean2 > 0.999))
      error('Cannot meet reverberation time with desired absorption_wall_scale, reduce contrast');
   end
   
%    % check whether we meet the mean abs coeefs again
%    k = [1 2; 3 1; 2 3; 3 2; 1 3; 2 1];  % (materials specified as [-z -y -x x y z])
%    surfs2 = prod(room.boxsize(k), 2);
%     sum(absmean2.*repmat(surfs2,1,length(absmean)))/S 

absmean = absmean2;
end
