function stDir = create_convex_hull(stDir)
    % Returns given struct with additional fields of convex hull calculation.
    % 
    % Usage:
    %  stDir = create_convex_hull(stDir)
    %
    % Input:
    %   stDir : struct
    %     Directional IR struct see new_dirstruct.m for more details. 
    %
    % Output:
    %   stDir : struct
    %     Directional IR struct with additional fields for convex hull.
    %     (Adds following fields:
    %     .vertices, .convex_hull, .centroids, .centroids_azim, .centroids_elev).
    %
    
%------------------------------------------------------------------------------
% RAZR engine for Mathwork's MATLAB
%
% Version 0.96.3
%
% Author(s): Siegfried Guendert
%
% Copyright (c) 2014-2023, Torben Wendt, Steven van de Par, Stephan Ewert,
% University of Oldenburg, Germany.
%
% This work is licensed under the
% Creative Commons Attribution-NonCommercial-NoDerivs 4.0 International
% License (CC BY-NC-ND 4.0).
% To view a copy of this license, visit
% http://creativecommons.org/licenses/by-nc-nd/4.0/ or send a letter to
% Creative Commons, 444 Castro Street, Suite 900, Mountain View, California,
% 94041, USA.
%------------------------------------------------------------------------------


    % xyz coordinates of azim eleve on unit sphere.
    [stDir.vertices(:, 1), ...
    stDir.vertices(:, 2), ...
    stDir.vertices(:, 3)] = sph2cart(stDir.azim/180*pi, stDir.elev/180*pi, 1);
    
    dtri = delaunayTriangulation(stDir.vertices);
    [stDir.convex_hull, vol] = convexHull(dtri);
    
    centroids = zeros(length(stDir.convex_hull), 3);
    for kk = 1:length(stDir.convex_hull)
        triangle = stDir.vertices(stDir.convex_hull(kk, :), :);
        centroid = mean(triangle, 1);
        centroid = centroid/norm(centroid);
        centroids(kk, :) = centroid;
    end
    stDir.centroids = centroids;
    [centr_azim_rad, centr_elev_rad, discard] = cart2sph( ...
            centroids(:, 1), centroids(:, 2), centroids(:, 3));
    stDir.centroids_azim = centr_azim_rad/pi*180;
    stDir.centroids_elev = centr_elev_rad/pi*180;
end
