function irOut = interpolate_directional_irs(dirs, weights, method)
    % Returns interpolated impulse response
    %
    % Usage:
    %   ir = interpolate_directional_irs(dirs, weights, method)
    %
    % Input:
    %   dirs : struct
    %     Containing N directional impulse responses.
    %     See `new_dirstruct.m` for field information.
    %   weights : array (N)
    %     Weighting factors for interpolation.
    %   method : str 
    %     Interpolation method. Options:
    %     {blend, delay_blend, magnitude_phase, magnitude_minimumphase}
    %     'blend': averages impulse responses in time domain
    %     'delay_blend': averages impulse responses in time domain und 
    %        uses average of stDir.IRdelay to delay the output response. 
    %        Expects the stDir.IR are delay compensated.
    %     'magnitude_phase': averages magnitude and phase in spectral domain
    %     'magnitude_minimumphase': averages magnitude spectra
    %        and creates response using minimum phase.  
    %
    % Returns:
    %   ir : array
    %     Interpolated impulse response.
    %
    % #lic:sg
    
    % SE some loop for different channels, better vectorize in the future
    IR = dirs.IR;
    if size(dirs.IR,3) > 1
        loop = size(dirs.IR,2);
        irOut = zeros(loop,size(dirs.IR,3)); % SE this whole code handles samples not in the first dimension, confusing
    else
        loop = 1;
        irOut = zeros(loop,size(dirs.IR,2));
    end
    
    for kk = 1:loop
        if loop > 1
            dirs.IR = squeeze(IR(:,kk,:));
        end
        
    % SE back to original single channel code
    irlen = size(dirs.IR, 2);
    switch method
        case 'blend'
            ir = weighted_sum(weights, dirs.IR);
            
        case 'delay_blend'
            % simple weighted average and weighted delay, recommended
            % if database has delay separated out from the IRs

            if isempty(dirs.IRdelay)
                error('delay_blend not applicable, no delays present');
            end

            % simple weighted average spectrum
            frf = fft(weighted_sum(weights, dirs.IR));

            fs = dirs.fs;
            delays = dirs.IRdelay(:);
            % weighted average delay in samples
            samples_delay = weighted_sum(weights, delays)/fs;
            
            % create a delaying spectrum from samples_delay
            delay_frf = exp(...
                -2i*pi*samples_delay ...
                * (0:irlen-1)./irlen*fs);
            
            mask = create_frf_mask(irlen);
            % apply delay in spectrum and transform back
            frf_delayed = (frf.*mask).*delay_frf;    
            ir = real(ifft(frf_delayed)); 

        case 'magnitude_phase'
            % interpolation of magnitude and phase response, most
            % recommened 
            frfs = fft(dirs.IR, irlen, 2);

            frf_abs = weighted_sum(weights, abs(frfs));
            frf_phase = weighted_sum(weights, unwrap(angle(frfs)));

            mask = create_frf_mask(irlen);
            ir = real(ifft((frf_abs.*mask).*exp(1i*frf_phase)));
            
        case 'magnitude_minimumphase'
            % interpolation of magnitude
            % with minimal phase
            frfs = fft(dirs.IR, irlen, 2);

            frf_abs = weighted_sum(weights, abs(frfs));
            
            % minimal phase from averaged absolute spectrum
            frf_phase = -imag(hilbert(log(frf_abs)));
            
            mask = create_frf_mask(irlen);
            ir = real(ifft((frf_abs.*mask).*exp(1i*frf_phase)));
    end
    
    % SE loop end
    irOut(kk,:) = ir;
    
    end
    
    % -SE
    
end

% subfcns
function y = weighted_sum(weights, x)
y = sum(weights(:).*x, 1);
end

function mask = create_frf_mask(irlen)

% SE support for uneven irlen
if round(irlen/2) == irlen/2
    mask = [1, 2*ones(1, irlen/2-1), 1, zeros(1, irlen/2-1)];
else
    mask = [1, 2*ones(1, ceil(irlen/2)-1), zeros(1, ceil(irlen/2)-1)];
end

end
