function [intersected, t, u, v] = intersect_ray_triangle(raydir, a, b, c, geoeps)
    % [intersected, t, u, v] = intersect_ray_triangle(raydir, a, b, c, geoeps)
    % Returns whether ray (raydir) intersects triangle (a, b, c) 
    % and parameters t, u, v
    %
    % Solves linear equation system using cramer's rule
    % [t,u,v]^T = 1/(-D*N) [N*T, E2*Q, -E1*Q]^T
    %
    % with
    % E1 = V1-V0
    % E2 = V2-V0
    % T = O-V0
    % N = E1 x E2
    % Q = -D x T
    % 
    % O is assumed to be at the origin of the coordinate system.
    %
    % Parameters
    % ----------
    % raydir : vector (3)
    %     Direction of the ray. (Symbol D)
    % a,b,c : vectors (3)
    %     Triangel Vertices. (Symbols V0 V1 V2)
    % geoeps : scalar
    %     Geometric accuracy.
    %
    % Returns
    % -------
    % intersected : bool
    %     Equals 1 if ray intersects triangle else 0.
    % t : scalar
    %     Distance from ray origin to the intersection point.
    % u, v : scalars
    %     Barycentric coordinates of the intersection point. 
    %     The third barycentric coordinate is w = 1-u-v.
    %
    % Reference: Ericson C., Real-time collision
    % detection, CRC Press, 2004. 
    % P. 190 ff. 
    % Ch. 5.3.6, is referencing Mueller Trumbore 1997
    % but gives slightly different solution with triangle normal 
    % compared to MT97.
    %
    % Code adapted from Stephan Ewert 2021
    %
    % Code refactoring Siegfried Guendert 2021

    % Validate input
    validate3d(raydir);
    validate3d(a);
    validate3d(b);
    validate3d(c);

    if geoeps < 0
        error("geoeps must be >= 0")
    end

    % Initialize output to enable early return if there is no intersection
    intersected = false;
    t = nan;
    u = nan;
    v = nan;

    % Triangle edge 1
    ab = b-a;

    % Triangle edge 2
    ac = c-a;

    % Compute the triangle normal
    % Note adapted from SE: could be cached (ab,ac as well) if tests 
    % against same triangle occur often. 
    % The normal has to be not normalized (speedup by 4%, SE 2021)
    % Note: writing out the cross product is faster than the 
    % matlab cross(ab, ac) function -> custom cross3d inline function is used. 
    tri_normal = cross3d(ab, ac);

    % Compute denominator determinant 
    denom = dot3d(-raydir, tri_normal);

    % update geoeps
    % for compatibility to old triangle intersection (SE 2021)
    geoeps = abs(denom) * geoeps; 

    % The triangle has two faces: back and front. 
    % The ray can hit from one of both sides or be parallel:
    ray_from_back = (denom > geoeps);
    ray_from_front = (denom < -geoeps);

    if ray_from_back
        % Triangle ray edge ap
        ap = -a;  % ap = p-a but p is assumed to be zeros.

        % Distance t to intersection
        t = dot3d(ap, tri_normal);

        if (t < 0)  % no intersect from back
            return
        end

        qvec = cross3d(-raydir, ap);

        % Parameter u (without division by denom)
        u = dot3d(ac, qvec);  

        if (u < -geoeps) || (u > denom+geoeps)  % no intersect from back
            return
        end

        % Parameter v (without division by denom)
        v = dot3d(-ab, qvec);

        if (v < -geoeps || u + v > denom+geoeps)  % no intersect from back
            return
        end
        
    elseif ray_from_front
        % Triangle ray edge ap
        ap = -a;  % ap = p-a but p is assumed to be zeros.

        % Distance t to intersection
        t = dot3d(ap, tri_normal);

        if (t > 0)  % no intersect from front
            return
        end

        qvec = cross3d(-raydir, ap);

        % Parameter u (without division by denom)
        u = dot3d(ac, qvec);  

        if (u > geoeps || u < denom-geoeps)  % no intersect from front
            return
        end

        % Parameter v (without division by denom)
        v = dot3d(-ab, qvec);

        if (v > geoeps  || u + v < denom-geoeps)  % no intersect from front
            return
        end

    else  % Ray is parallel to triangle
        return
    end

    % Ray intersects triangle. 
    intersected = 1;

    % Perform delayed division for parameters t, u, v.
    nomin = 1/denom;
    t = t * nomin;
    u = u * nomin;
    v = v * nomin;

    function d = dot3d(a, b)
        % Returns the dot product for 3 dim vectors a, b.
        % Parameters
        % ----------
        % a : vector (3)
        % b : vector (3)
        %
        % Returns
        % -------
        % d : scalar
        d = a(1)*b(1) + a(2)*b(2) + a(3)*b(3);
    end

    function c = cross3d(a, b)
        % Returns the cross product for 3 dim vectors a, b.
        % Parameters
        % ----------
        % a : vector (3)
        % b : vector (3)
        %
        % Returns
        % -------
        % c : vector (3x1)
        c = [ 
            a(2)*b(3) - a(3)*b(2);
            a(3)*b(1) - a(1)*b(3);
            a(1)*b(2) - a(2)*b(1);];
    end

    function validate3d(a)
        if numel(a) ~= 3
            error("Given vector must have 3 elements.")
        end
    end
end
