function directional_ir = pick_directional_ir(azim, elev, stDir, interpolation_method)
    % directional_ir = pick_directional_ir(azim, elev, stDir, method)
    % Returns a directional impulse response (DIR)
    %
    % Parameters
    % ----------
    % azim : scalar
    %   Target azimuth angle in degree.
    % elev : scalar
    %   Target elevation angle in degree.
    % stDir : struct
    %   Directional impulse response struct.
    %   See new_dirstruct() for details.
    % interpolation_method : str
    %   Picking method.
    %   'nearest': returns nearest DIR to given angles.
    %   'blend' : returns an interpolated DIR from the 
    %       three DIRs located on a triangle around given angles.
    %   'delay_blend': weighted average with delays from stDir.IRdelays, 
    %   'magnitude_phase': weighted average of magnitude and phase.
    %   'magnitude_minimumphase': averages magnitude spectra
    %      and creates response using minimum phase.  
    %
    
    switch interpolation_method
        case 'nearest'
            idx = find_nearest_coordinates_on_unitsphere( ...
                azim, elev, stDir.azim, stDir.elev);
            directional_ir = stDir.IR(idx, :, :); % SE added dimension for channels
            return
        otherwise
            idx_centroids_by_distance = find_nearest_coordinates_on_unitsphere( ...
                azim, ...
                elev, ...
                stDir.centroids_azim, ...
                stDir.centroids_elev, ...
                length(stDir.centroids));
            convex_hull_by_distance = stDir.convex_hull(idx_centroids_by_distance, :);

            [dirvec(1), dirvec(2), dirvec(3)] = sph2cart(azim/180*pi, elev/180*pi, 1);
            [idx_hull, t, u, v] = find_triangle_on_hull( ...
                dirvec, ... 
                stDir.vertices, ...
                convex_hull_by_distance, ...
                stDir.geoeps);
            if isnan(idx_hull)
                error('No hull found for dirvec.')
            end
            idx_triangle = convex_hull_by_distance(idx_hull, :);
            dirs = get_dirs_by_index(stDir, idx_triangle);
            weights = [1-u-v, u, v];
            directional_ir = interpolate_directional_irs(dirs, weights, interpolation_method);
    end    
    
    function dirs = get_dirs_by_index(stDir, idx)
        dirs.IR = stDir.IR(idx, :, :); % SE added dimension for channels
        if any(stDir.IRdelay)
            dirs.IRdelay = stDir.IRdelay(idx);
        end
        if length(stDir.fs) > 1
            dirs.fs = stDir.fs(idx);
        else
            dirs.fs = stDir.fs;
        end
    end
end  % function
