function op = get_default_options
% GET_DEFAULT_OPTIONS - Returns struct containing all default options for RAZR
%
% Usage:
%   op = GET_DEFAULT_OPTIONS
%
% See also: RAZR, SOP, GET_DEFAULT_OPTIONS_*

%------------------------------------------------------------------------------
% RAZR engine for Mathwork's MATLAB
%
% Version 0.96.3
%
% Author(s): Torben Wendt
%
% Copyright (c) 2014-2023, Torben Wendt, Steven van de Par, Stephan Ewert,
% University of Oldenburg, Germany.
%
% This work is licensed under the
% Creative Commons Attribution-NonCommercial-NoDerivs 4.0 International
% License (CC BY-NC-ND 4.0).
% To view a copy of this license, visit
% http://creativecommons.org/licenses/by-nc-nd/4.0/ or send a letter to
% Creative Commons, 444 Castro Street, Suite 900, Mountain View, California,
% 94041, USA.
%------------------------------------------------------------------------------


%% DO NOT EDIT THIS FILE!
%%
%% This is the place where all default options are stored. If you'd like to synthesize BRIRs using
%% other options, please create a new structure specifying your desired options and pass it to RAZR.
%% In your custom options structure, only those fields need to be specified, whose values differ
%% from the default. All other fields will automatically be taken from the defaults.
%% For an example, see EXAMPLE_OPTIONS.M
%%

%% General
op.debugMode = 0;           % enable debug modus

op.rirname = '';            % Name of RIR. If empty, the name of the room will be taken, if exists
op.fs = 44100;              % Sampling rate in Hz
op.len = [];                % RIR length in samples; if empty, it will be set to the estimated T60,
                              % (mean or max over frequencies, see op.len_rt_measure) multiplied
                              % with op.len_rt_factor
op.len_max = 882e3;         % If op.len is larger than this value, the RIR length will
                              % automatically be reduced to this value (a warning will be displayed)
op.len_rt_factor = 1.0;     % See description of op.len
op.len_rt_measure = 'max';  % 'max' or 'mean', see description of op.len
op.SPL_at_0dBFS = 100;      % SPL at 0 dB full scale
op.SPL_source = 100;        % SPL at 1 m distance to source
op.pseudoRand = 1;          % Use fixed seeds for all random generator initialisations?
op.legacy_rand = 0;         % Use legacy random-number-generator seeds?
op.seed_shift = 0;          % If op.pseudoRand = true, this number will be added to all rng seeds
op.legacy_distce_atten = 0; % Use legacy smooth transition to constant gain for distances <= 1 ?
op.rt_estim = 'eyring';     % RT-estimation measure ('eyring' or 'sabine'), see also ESTIMATE_RT

op.enableSR = [1 0];        % Use smearing component? Vector [SRism, SRfdn], controls for ISM, FDN
op.distSR = 1;              % Use distance-based smearing coefficients similar to liveRAZR implementation?
op.typeSR = 2;              % Type of smearing
                              % 2: gamma like, cacade of 4 all-passes
                              % 6: original Schroeder reverberator, cacade of 4 all-passes
% CK 21-08-11
op.filtCreatMeth = 'jt';    % Method for reflection/absorption filter synthesis, possible options:
                              % 'cq': composedPEQ, 'sh': shEQ, 'cs': composedShelving, 'yw': yulewalk
op.diffrFiltMeth = 'comsh'; % Method for diffraction filter creation. Possible options:
                              % 'comsh': Comb and lowpass shelving filter
                              % 'udfa_basic': simplified aperture handling
                              % based on Kirsch and Ewert (2012), Ewert
                              % (2022), and subm 2023 paper

op.jp_reproduction_numerics = 0; % Use numerical tweaks to produce the same numbers as in JP's work

% CK 20-04-03
op.fltMode = 'smart';       % conv - convolution of coefficients leads to high order single filter, may become instable for many bands
                            % casc - cascaded (safest, but slowest option)
                            % smart - smart cascade, partly convolved,
                            %          partly cascaded
                            % SOS - 2nd order sections, experimental and
                            % not fully implemented, use with care

%% Spatialization

op.spat_mode = 'shm';       % Spatialization modes for {early, late} BRIR part, possible values:
                              % 'diotic', 'hrtf', 'ild' (broadband), 'shm' (spherical head model),
                              % 'array' (loudspeaker array, in development, not supported yet).
                              % If only one key string is specified, it will be used for both early
                              % and late RIR part.
% HRTF/SHM:
op.hrtf_database = '';      % Key string for HRTF database. See BASE/HRTF/APPLY_HRTF.M or README
op.hrtf_options = struct;   % Struct containing options being passed to APPLY_HRTF
op.hrtf_filtfunc = 'fftfilt'; % Function to be chosen to filter with HRTFs ('fftfilt', 'filter')
op.spat_limit_filtrange = 0;% Limit HRTF/SHM filtering of early refl. to shorter ranges?
op.force_loading_hrtf = 0;  % Reload HRTF data and don't use persistent variables in get_hrtf_dbase

% interpolation_method : str
%   Picking method.
%   'nearest': returns nearest DIR to given angles.
%   'blend' : returns an interpolated DIR from the 
%       three DIRs located on a triangle around given angles.
%   'delay_blend': weighted average with delays from stDir.IRdelays, 
%   'magnitude_phase': weighted average of magnitude and phase.
op.hrtf_interpolation_method = 'nearest';  % see pick_directional_ir.m

op.shm_warpMethod = 1;      % For sperical head model: Prewarp theta to better account for smaller 
                              % changes in attenuation at small theta
                              % warpMethod = 0 --> do not warp
                              % warpMethod = 1 --> exponent transform warp
                              % warpMethod = 2 --> alternative warping

% Loudspeaker array:
op.array_pos = [];          % Coordinates of the speakers relative to the listeners,
                              % if spat_mode is 'array'.
op.array_pos_rec = [];        % keep the original receiver coords for 
                              % headphone spatialization
op.array_render = 'nearest';  % Rendering mode for op.spat_mode == 'array', possible values:
                              % 'nearest': Map sources to nearest speaker in projection onto
                              %            unit sphere
                              % 'vbap':    Apply vector base amplitude panning (vbap package must
                              %            be downloaded! See instruction in EXAMPLE_ARRAY.M)
op.array_distance_comp = 1; % Option to set distance compensation in render_array with no post spat - 27.03.2020 HS                              
op.enable_post_spat = 0;    % For array rendering: Additional binaural synthesis of array signal?
op.post_spat_mode = 'shm';  % Rendering method for array binaural synth.: 'shm', 'hrtf', 'diotic'
op.vbap_color_compen = 0;   % Apply coloration compensation within VBAP?
op.array_TCelsius = 20;     % Temperature (in degree Celsius) of the playback room, where the
                              % loudspeaker array is placed (not to be confused with the
                              % temperature of the simulated room, which is specified in the
                              % room structure). This option will be ignored if
                              % op.jp_reproduction_numerics == true.
                              
op.array_plotPwr = 1;        % CK 20-04-03: Automatically plot power distribution on array when this spatialisation is chosen.

%% SE 13.02.2020
op.hrtf_options.mk2_legacy_angle_conversion = 0;  % unknown option for mk2 database used in default_options_jp 
                                                  % and required in   pick_hrir_mk2.m                              

%% Returned data

op.return_rir_parts = 0;    % Return ir.sig_direct, ir.sig_early and ir.sig_late?
op.return_ism_data = 0;     % Return geometric data of ISM?
op.return_ism_sigmat = 0;   % Return monaural ISM output (one image source per matrix channel)?
op.return_op = 0;           % Return applied options structure (as additional field of ir)?
op.verbosity = 0;           % Verbosity level (messages printed to screen)
                              %   0: no messages,
                              % > 0: display calculation duration,
                              % > 1: display calculation durations for ISM and FDN,
                              % > 2: display durations for ISM orders, too

%% Early-reflection rendering (image source model)

op.ism_order = 3;               % Max. image source order or vector [minOrder, maxOrder]
op.ism_only = -1;               % Use only image source model? If not -1, this number overwrites
                                  % also op.ism_order and op.fdn_enabled (to 0). op.verbosity is set
                                  % to 3, if not specified.
op.ism_enableReflFilt = 1;      % Enable reflection filters?
op.ism_enable_scattering = 1;   % Enable scattering module?
op.ism_diffu_specu_ratio = 1.0; % Energy ratio of diffuse and specular reflections (1.0...2.0)
op.ism_enable_timespread = 1;   % Enable time spreading of diffuse reflections?
op.ism_enableAirAbsFilt = 1;    % Enable air absorption filters? (Set to -1 for excessive mode)
op.ism_enableToneCorr = 1;      % Enable tone correction filter?
op.ism_enableDiffrFilt = 1;     % Enable diffraction filtering for outside-lying receiver?
op.ism_diffr_mc_output = 0;     % Enable diffr. filtering on multichannel ISM output (which will be
                                  % the FDN input) (only enabled, if op.ism_enableDiffrFilt = 1)
op.ism_enableBP = 1;            % Enable global bandpass?
op.legacy_bp = 0;               % Use legacy edge frequencies for bandpass?
op.ism_refl_gain = 0;           % Gain (in dB), applied on all image sources
op.ism_filtlen = 4410;          % Shorten reflection and other filters to this length to save
                                  % computation
op.ism_jitter_type = 'cart';    % ISM jitter in 'cart' or 'sph' coordinates. 'cart_legacy' restores
                                  % behavior from RAZR 0.91 and before.
op.ism_jitter_factor = 0.05;    % Random factor on IS positions. May be a vector [azim, elev, dist]
                                  % if op.ism_jitter_type is 'sph'
op.ism_rand_start_order = 2;    % ISM order at which the random jitter starts.
op.ism_norand_if_diffr = 1;     % Set op.ism_jitter_factor to zero, if diffraction is applied?
op.ism_discd_directions = [];   % Directions ([-3, -2, -1, +1, +2, +3] <-> [-z -y -x +x +y +z]), for
                                  % which all image sources are discarded. Automatically set, if
                                  % source or receiver lies outside room.
op.ism_drct_snd_rec_outs = 0;   % If true, render direct sound, if the receiver is outside the room
                                  % (For coupled rooms, the direct sound must rendered only once)
                                  
op.ism_enableSourceDir = 0;     % Enable source directionality filtering?
op.ism_order_avg = op.ism_order+1; % Use avg src directivity starting from n-th ISM order
                                   % (op.ism_order+1 means no avg. filtering is applied)
                                   % ir_avg_hs_filter.mat was precalculated
                                   % with 
op.dir_mode = 'hs_filter';        % 'hs_filter' is a simple source directivity model based on SHM, 
                                % see Ewert, Buttler, and Hu (2021), use 'sofa' to use a
                                % sofa file format database
op.dir_src_radius = 0.08;       % radius of src (head);
op.dir_theta0 = 180;            %(minimum of hs_filter at 180 deg, 150 in Zoelzer's DAFX) 

op.dir_database = '';

% source directivity options for sofa directional impulse responses

% interpolation_method : str
%   Picking method.
%   'nearest': returns nearest DIR to given angles.
%   'blend' : returns an interpolated DIR from the 
%       three DIRs located on a triangle around given angles.
%   'delay_blend': weighted average with delays from stDir.IRdelays, 
%   'magnitude_phase': weighted average of magnitude and phase.
op.dir_interpolation_method = 'nearest';  % see pick_directional_ir.m

% instead of defining a sofa file in cfg, you can directly provide a dir
% struct in 'op.dir_struct'. 
% For loading a directivity database from sofa you can use the function
% op.dir_struct = load_source_directivity_sofa();
op.dir_struct = nan;


%% Late reverb (FDN)

op.fdn_enabled = 1;             % Enable FDN?
op.fdn_legacy_input = 0;        % Generate FDN input as in 2014 (differs only if op.ism_order == 0)
op.fdn_numDelays = 24;          % Number of FDN delays (thus: channels). [6 12 24 48 96] must be equal or higher op.fdn_numDownmixDirections
                                % use 12 vor computationally sensitive
                                % applications, otherwise: 24
op.fdn_fmatrix = 'randOrth';    % Feedback matrix: 'eye' (unit matrix), 'house' (Householder),
                                  % 'hadam' (Hadamard), 'randOrth' (random orthogonal), also
                                  % direct input of a square matrix is possible
op.fdn_enable_apc = 1;          % Enable allpass cascade to smear FDN input?
op.fdn_enableAbsFilt = 1;       % Enable absorption filters?
op.fdn_enableReflFilt = 1;      % Enable reflection filters?
op.fdn_enableBP = 0;            % Enable global bandpass?
op.fdn_hrtf_on_cube = 1;        % Virt. reverb sources on cube surface around receiver? (Otherwise
                                  % on actual walls)
op.fdn_smart_ch_mapping = 0;    % Sort early refl. to FDN channels of similar directions
op.fdn_legacy_angles = 0;       % If true, use FDN spatialization angles from Versions < 0.94

op.fdn_twist_directions = 0;    % If true, use twisted spatialization angles (only available for
                                  % op.fdn_legacy_angles == true)
op.fdn_delay_stretch = 1.0;     % Factor to stretch delays
op.fdn_delays_choice = 'diag';  % Choice of FDN delays: 'roomdim2014': after Wendt et al. (2014),
                                  % JAES 62, 'diag', 'hyp', 'hyp2', 'lin': improved methods;
                                  % also direct input of a vector is possible.
op.fdn_delay_criterion = 'jot'; % Criterion for sufficient delay mode density; 'jot' or 'smith'.
op.fdn_blend_reflcoeffs = 0;    % Blend reflection coeffs?
op.fdn_numDownmixDirections=[]; % If empty, no downmixing will be performed, must be equal or smaller than op.fdn_numDownmixDirections [6 12 24 48 96]
op.fdn_reflBeforeDownmix = 1;   % Apply reflection filtering before downmix? Otherwise after
op.fdn_door_idx_to_shift_angles = 0; % Idx of door to neighbour room, for which reverb is rendered.
                                  % Set to 0, if no neighbour room exists
op.fdn_return_mc_output = 0;    % Return the multichannel FDN output? (Field ir.sig_late_mc)
op.fdn_enable_spat_div = 1;     % Coupled rooms only: Map part of neighbour-room reverb to reverb-
                                  % cube of receiver room?
                                  
%% Plots

op.plot_reflFilters = 0;        % Reflection filters (ISM)
op.plot_diffr_points_ism = 0;   % Diffraction points

op.plot_absFilters = 0;         % Absorption filters (FDN), 1 = filter stages, 2 = filter cascade
op.plot_filters_fdn_bin = 0;    % Reflection filters (FDN)
op.plot_shifted_angles = 0;     % Shifted virt. reverb sources (see op.fdn_door_idx_to_shift_angles)
op.plot_vbap_arrangement = 0;   % Plot spheres with triangles of loudspeakers for vbap
