function stDir = load_source_directivity_sofa(fpath, save_convex_hull)
    % Returns directivity struct stDir from sofa file with additional convex hull data.
    %
    % Parameters
    % ----------
    % fpath : str
    %   path to sofa file
    % save_convex_hull : logical
    %   If true, convex hull data will be saved aside the sofa file.
    %
    % Returns
    % -------
    % stDir : struct
    %   Directivity struct.
    %
    
%------------------------------------------------------------------------------
% RAZR engine for Mathwork's MATLAB
%
% Version 0.96.3
%
% Author(s): Siegfried Guendert
%
% Copyright (c) 2014-2023, Torben Wendt, Steven van de Par, Stephan Ewert,
% University of Oldenburg, Germany.
%
% This work is licensed under the
% Creative Commons Attribution-NonCommercial-NoDerivs 4.0 International
% License (CC BY-NC-ND 4.0).
% To view a copy of this license, visit
% http://creativecommons.org/licenses/by-nc-nd/4.0/ or send a letter to
% Creative Commons, 444 Castro Street, Suite 900, Mountain View, California,
% 94041, USA.
%------------------------------------------------------------------------------

    persistent stDirDb
    
    if isempty(stDirDb)
        
        try
            % SE we might have already started for SOFA for HRTF
            SOFAstart('silent');
        end
        
        stSofa = SOFAload(fpath);
        stDirDb = new_dirstruct();
        stDirDb.fpath_sofa = fpath;
        stDirDb.azim = stSofa.ReceiverPosition(:, 1);
        stDirDb.elev = stSofa.ReceiverPosition(:, 2);
        stDirDb = retrieve_convex_hull(stDirDb, save_convex_hull);
        stDirDb.IR = squeeze(stSofa.Data.IR);
        stDirDb.fs = stSofa.Data.SamplingRate;
        stDirDb.IRdelay = stSofa.Data.Delay;
    end
    
    stDir = stDirDb;
    
end

function stDir = retrieve_convex_hull(stDir, save_hull)
    % Returns stDir with additional fields of convex hull.
    stDir.fpath_convex_hull = strcat(stDir.fpath_sofa, '.chull.mat');
    fchull = dir(stDir.fpath_convex_hull);
    fsofa = dir(stDir.fpath_sofa);
    if ~isempty(fchull)
        chull_is_newer_than_sofa = fchull.datenum > fsofa.datenum;
    else
        chull_is_newer_than_sofa = false;
    end
    if isfile(stDir.fpath_convex_hull) && chull_is_newer_than_sofa
        stDir = load_convex_hull_data(stDir);
    else
        stDir = create_convex_hull(stDir);
        if save_hull
            save_convex_hull_data(stDir);
        end
    end
end

function stDir = load_convex_hull_data(stDir)
    stHull = load(stDir.fpath_convex_hull);
    stDir.convex_hull = stHull.convex_hull;
    stDir.vertices = stHull.vertices;
    stDir.centroids = stHull.centroids;
    stDir.centroids_azim = stHull.centroids_azim;
    stDir.centroids_elev = stHull.centroids_elev;
end

function save_convex_hull_data(stDir)
    stHull.fpath_sofa = stDir.fpath_sofa;
    stHull.convex_hull = stDir.convex_hull;
    stHull.vertices = stDir.vertices;
    stHull.centroids = stDir.centroids;
    stHull.centroids_azim = stDir.centroids_azim;
    stHull.centroids_elev = stDir.centroids_elev;
    save(stDir.fpath_convex_hull, '-struct', 'stHull')
end
