function rec_point_on_src = get_rec_point_on_rotated_src_basis(src_az, src_el, rec_point)
% Returns karthesian coordinates of receiver point in the rotated source
% coordinate system
% Projection of rec_point into the src coordinate system.
%
% Parameters
% src_az : scalar
%    Azimuth angle of source.
% src_el : scalar
%    Elevation angle of source.
% rec_point : array(3)
%    Karthesian coordinates of point.
%
% Reuturns
% rec_point_on_src : array(3)
%    Karthesian coordinates of point in src coordinate system.

if nargin == 2
    rec_point = [1,0,0];
end

src_az = src_az/180*pi;
src_el = src_el/180*pi;

% Rotation for orthotogonal basis of src coordinate system
Ry = [cos(src_el) 0 sin(src_el); 0 1 0; -sin(src_el) 0 cos(src_el)]';
Rz = [cos(src_az) -sin(src_az) 0; sin(src_az) cos(src_az) 0; 0 0 1];

% Construct rotated basis or src coordinate system
src_front = Rz*(Ry*[1,0,0]');
src_left = Rz*(Ry*[0,1,0]');
src_up = Rz*(Ry*[0,0,1]');


% Calculate unit-vectors of rotated basis 
% and scalar products with rec_point
esf = src_front/norm(src_front);
xsp = rec_point*esf;

esl = src_left/norm(src_left);
ysp = rec_point*esl;

esu = src_up/norm(src_up);
zsp = rec_point*esu;
rec_point_on_src = [xsp, ysp, zsp];
