clear
close all

%% Test Parameters
NUMBER_OF_TESTCASES = 10;
NFFT = 2*8192;
VERBOSE = false;

%% Create an anechoic chamer
room.name = 'Anechoic';
% Room dimensions [x, y, z] in meters:
room.boxsize = [8.5, 7, 4];
% Boundary absorption alpha
room.materials = zeros(1, 6) + 0.999;
% Frequency base (in Hz) for absorption coefficients
room.freq = [250 500 1e3 2e3 4e3 8e3];    
% Position [x, y, z] of sound source in meters:
room.srcpos = [3.25, 3.5, 2];
room.srcdir = [0,0];  % this varable will be changed in this test
% Position [x, y, z] of receiver in meters:
room.recpos = [4.25, 3.5, 2];
% Orientation [azimuth, elevation] of receiver in degrees
room.recdir	= [180, 0];    
room.TCelsius = 20;
%scene(room)

%% Load source directivity database
cfg = razr_cfg_user();
stDir = load_source_directivity_sofa(cfg.dir_path__sofa, true);
% This test was developed using the 
% '... Genelec8020_directivity_IR_front_pole.sofa' database.

% Create options for the simulation
op = get_default_options();
op.spat_mode = 'diotic';  % omnidirectional receiver
op.ism_enableSourceDir = true;  % sourc with directivity filtering
op.dir_db = 'sofa';  
op.dir_interpolation_method = 'nearest';
op.return_rir_parts = true;  % we need the late IR part separately
op.fdn_enabled = false;
op.enableSR = false;
op.ism_order = 0;
op.ism_enableAirAbsFilt = false;
op.ism_enableDiffrFilt = false;
op.ism_enableToneCorr = false;
op.ism_enableReflFilt = true;
op.ism_enable_scattering = false;

%% Test cases
source_orientataions_az_el = [
    % Special cases
    [90, 45];
    [0, 0];
    [180, 0];
    [90, 0];
    [rand(NUMBER_OF_TESTCASES, 1)*360, zeros(NUMBER_OF_TESTCASES, 1)];  % elev=0
    % Create uniformly distributed coordinates pairs
    [rand(NUMBER_OF_TESTCASES, 1)*360, rand(NUMBER_OF_TESTCASES, 1)*180-90]
    ];

numerrors = 0;
numsuccess = 0; 

for ii = 1:length(source_orientataions_az_el)
    source_orientation_azim = source_orientataions_az_el(ii, 1);
    source_orientation_elev = source_orientataions_az_el(ii, 2);
    rec_xyz = [1, 0, 0];
    room.srcdir = [source_orientation_azim, source_orientation_elev];
    %scene(room)
    ir_razr = razr(room, op);
    [azim_expected_ir, elev_expected_ir] = get_rec_sphcoords_on_rotated_src_basis(...
        source_orientation_azim, source_orientation_elev, rec_xyz);
    [vx,vy,vz] = sph2cart(azim_expected_ir/180*pi, elev_expected_ir/180*pi, 1);

    idx_expected = knnsearch(stDir.vertices, [vx, vy, vz], "K", 1);
    idx = find_nearest_coordinates_on_unitsphere( ...
        azim_expected_ir, elev_expected_ir, stDir.azim, stDir.elev);

    ir_expected = stDir.IR(idx_expected, :);
    assert(all(ir_razr.fs == stDir.fs), 'Samplerates must be equal.')
    spec_razr = fft(ir_razr.sig_direct(:,1), NFFT);
    spec_expected = fft(ir_expected(:), NFFT);
    spec_diff_mse = mean(...
        (abs(fft(ir_razr.sig_direct(:,1), NFFT)) ...
        - abs(fft(ir_expected(:), NFFT))).^2);
    
    if VERBOSE
        disp('Source Orientation:')
        disp(source_orientation_azim)
        disp(source_orientation_elev)
        disp('Expected IR coordinates:')
        disp(azim_expected_ir)
        disp(elev_expected_ir)
        disp('Indexes knn vs. dist on sph')
        disp(idx_expected)
        disp(idx)
        figure()
        plot(abs(spec_razr))
        hold on
        plot(abs(spec_expected), '--')
        disp('Abs spec diff MSE:')
        disp(spec_diff_mse)
    end
    if spec_diff_mse < 1e-6
        fprintf('OK, MSE:%e\n', spec_diff_mse)
        numsuccess = numsuccess + 1;
    else
        fprintf('FAILED, MSE:\n', spec_diff_mse)
        numerrors = numerrors + 1;
    end
    
end
fprintf('NUM ERRORS: %d\n', numerrors)
fprintf('NUM SUCCESS: %d\n', numsuccess)

