clear
close all

%% Test Source Directivity Feature in RAZR with an echoic chamber scenario

% The simulated room is an echoic chamber defined in get_room_echoic.m.
% A source, with a directivity pattern, and an omnidirectional receiver are
% placed in the room.

% The source in the room is using a sofa directivity database
% (Source Directivity Feature in RAZR to be tested).

% The receiver is defined as a diotic receiver without any spatialization.
% Hence it can be understood as an omnidirectional receiver.
% Air absorption is switched of for this simulation.
%
% The idea is to compare the spectral power at the omni directional 
% receiver with the spectral power of the source.
% Whereby an average power spectrum over all directions of the source 
% directivity is calculated. Only the late reverb at the receiver is
% taken into account.
% Resulting power spectra are visulized (they are biased to be 0 dB @ 1 kHz).

%% Load source directivity database
cfg = razr_cfg_user();
stDir = load_source_directivity_sofa(cfg.dir_path__sofa, true);
% This test was developed using the 
% '... Genelec8020_directivity_IR_front_pole.sofa' database.
% Where the measurement points on the sphere around the loudspeaker are not 
% equally distributed! --> Some weighting is needed to get equal 
% contribution of all directions.

figure()
hp = patch('Vertices', stDir.vertices, 'Faces', stDir.convex_hull);
set(hp,'FaceColor', 'none', 'EdgeColor', 'black', 'EdgeAlpha', 0.1);
daspect([1 1 1])
view(3);
camlight
lighting gouraud
xlabel('x')
ylabel('y')
zlabel('z')
view(65,-20)
title('Spatial Sampling Mesh of Source Directivity Database')
savefig('test_srcdir_echoic.spatial_sampling_sphere.fig')
saveas(gcf(), 'test_srcdir_echoic.spatial_sampling_sphere.png')

%% Source Directivity average spectrum of equally weighted sphere.
% Calculate one sided fourier spectrum of each impulse response from the
% directivity database:
spec_srcdir = fft(stDir.IR,[], 2);
spec_srcdir = spec_srcdir(:, 1:end/2+1);
spec_srcdir(:, 2:end-1) = 2 * spec_srcdir(:, 2:end-1);
nfft = size(stDir.IR, 2);

% Calculate the average spectrum over all directions (no weighting here)
spec_srcdir_sq_sum = sum(abs(spec_srcdir).^2, 1);
spec_srcdir_rms = sqrt(spec_srcdir_sq_sum/size(stDir.IR, 1));

% Calculate the average spectrum over all directions,
% whereby each spectrum is weighted by the distance to its nearest neighbor.
% So dense sampled regions have a smaller weight than sparse sampled
% regions.
weights = get_vertices_weights(stDir.vertices);
spec_srcdir_weighted = weights .* spec_srcdir;
spec_srcdir_weighted_sq_sum = sum(abs(spec_srcdir_weighted).^2, 1);
spec_srcdir_weighted_rms = sqrt(spec_srcdir_weighted_sq_sum/size(stDir.IR, 1));

% create a frequency vector for source directivity spectra
freq_spec_srcdir = stDir.fs * (0:nfft/2) / nfft;

%% Set up echoic room and 
%% create echoic room response spectrum of late reverb
room.name = 'Echoic';
% Room dimensions [x, y, z] in meters:
room.boxsize = [8.5, 7, 4];
% Frequency base (in Hz) for absorption coefficients
room.freq = [125 250 500 1e3 2e3 4e3 8e3 16e3];
room.materials = zeros(1, length(room.freq)) + 0.001;
% Position [x, y, z] of sound source in meters:
room.srcpos = [4.25, 3.5, 2];
room.srcdir = [180,0];
% Position [x, y, z] of receiver in meters:
room.recpos = [2.25, 2.5, 2];
% Orientation [azimuth, elevation] of receiver in degrees
room.recdir	= [0, 0];
room.TCelsius = 20;
scene(room);
savefig('test_srcdir_echoic.scene.fig')
saveas(gcf(),'test_srcdir_echoic.scene.png')

% Create options for the simulation
op = get_default_options();
op.spat_mode = 'diotic';  % omnidirectional receiver
op.ism_enableSourceDir = true;  % sourc with directivity filtering
op.dir_db = 'sofa';  
op.return_rir_parts = true;  % we need the late IR part separately
op.ism_enableAirAbsFilt = false;
op.ism_enableDiffrFilt = false;
op.ism_enableToneCorr = false;
op.ism_enableReflFilt = true;
op.ism_enable_scattering = false;

op.dir_struct = stDir;  % Source Directivity Database was loaded above

ir_dir = razr(room, op);

% ir without direct and early reflections:
ir_dir_no_direct = ir_dir.sig_late(:,1);  % just one channel

% power spectrum of late reverb
spec_ir_dir_no_direct = pwelch(ir_dir_no_direct, nfft, nfft/2, nfft);
freq_spec_ir_dir_no_direct = ir_dir.fs*(0:nfft/2) / nfft;

%% Make spectral power plot
figure()

% Curves will be biased to have the level at 1kHz equal to 0 dB(FS)
idx_1kHz = knnsearch(freq_spec_ir_dir_no_direct(:), 1000);

% Echoic Room (late reverb), smoothed:

% % Create a 1/5th octave smoothing matrix with rect. shaped window.
% % Smooth the absolute spectrum (THIS IS NO COMPLEX SMOOTHING).
% spec_ir_smoothed = complexsmooth.smooth( ...
%     spec_ir_dir_no_direct, ...
%     complexsmooth.half_rel_bw_octave(5), ...
%     1);
spec_ir_smoothed = smooth(spec_ir_dir_no_direct, 23, 'sgolay');
semilogx(freq_spec_ir_dir_no_direct,...
    10*log10(abs(spec_ir_smoothed)+eps)...
    - 10*log10(spec_ir_smoothed(idx_1kHz)), 'b')

hold on

% Echoic Room (late reverb):

semilogx(freq_spec_ir_dir_no_direct,...
    10*log10(abs(spec_ir_dir_no_direct)+eps)...
    - 10*log10(spec_ir_dir_no_direct(idx_1kHz)),...
    'color', [0.9, 0.9, 0.99])
    

idx_1kHz = knnsearch(freq_spec_srcdir(:), 1000);

% Spatial Sample Distance Weighted Source Hull:

semilogx(...
    freq_spec_srcdir, ...
    20*log10(abs(spec_srcdir_weighted_rms)+eps)...
    - 20*log10(spec_srcdir_weighted_rms(idx_1kHz)), 'r')


% Unweighted Source Hull:

semilogx(...
    freq_spec_srcdir, ...
    20*log10(abs(spec_srcdir_rms)+eps)...
    - 20*log10(spec_srcdir_rms(idx_1kHz)), 'color', [0.99, 0.65, 0.65])

grid on
legend({ ...
    'Echoic Room (late reverb), smoothed', ...
    'Echoic Room (late reverb)', ...
    'Source Hull (weighted by eucl. dist. to nn.)', ...
    'Source Hull (unweighted)'})

% Rearrange order of plots and legend:
h = get(gca,'Children');
set(gca,'Children', [h(4),h(2) h(1), h(3)])
legend([h(2), h(1), h(4), h(3)])
ylim([-10, 10])
xlim([50, stDir.fs/2-50])
xlabel('Frequency /Hz')
ylabel('Level /dB(FS)')
title('Test RAZR Source Directivity Feature with echoic room')
savefig('test_srcdir_echoic.spectra.fig')
saveas(gcf(), 'test_srcdir_echoic.spectra.png')
