% filter design for limited size occluder reflection and aperture
% diffraction based on
% Rindel, J. H. (1986). Attenuation of Sound Reflections Due to Diffraction, Aalborg, Denmark. Presented at the Nordic Acoustical Meeting
% Rindel, J. H. (2005). Reflection of sound from finite-size plane and curved surfaces, Journal of the Acoustical Society of America, 118, 20162016.

% verts has to be a rectangular object with ordered verts
% normal can be in the one or other direction, we always assume a twosided
% object

% Stephan Ewert 2021

function [b,a] = limSizeDiffractionFilter( src, rec, verts, plane, isImageSrc, isAperture, c, fs )

% % some test values
% src = [0 0 -1];
% rec = [0 40 10];
% verts = [-1 -1 0; 1 -1 0; 1 1 0; -1 1 0]*0.5; % square
% verts = [-1 -2 0; 1 -2 0; 1 2 0; -1 2 0]*0.5; % typical door
% 
% % plate as in Measurements of Rindel 2005 talk
% verts = [-1.3 -1 0; 1.3 -1 0; 1.3 1 0; -1.3 1 0]*0.5; 
% src = [-1 0 -2];
% rec = [1 0 2];
% src = src/norm(src)*3;  % Rindel 2005 talk
% rec = rec/norm(rec)*3;  % Rindel 2005 talk
% src = src + [0 0.0 0]; % offset Rindel 2005 talk
% rec = rec + [0 0.0 0];
% 
% planeNormal = [0 0 1];
% isImageSrc = true;
% isAperture = false;
% 
b = [];
a = [];

if isAperture && isImageSrc
   % no image src for an aperture, goodbye
   return; 
end

% get plane distance
planeDist = plane.dist;
planeNormal = plane.normal;
%planeDist = verts(1,:)*plane.normal';

% get main axis of rectangle (all these were actually in the original refl
% plane definition but we keep the code general for triangles
sa = verts(2,:)-verts(1,:);
sb = verts(3,:)-verts(2,:);
origin = verts(1,:)+sa/2+sb/2; % center origin

aLen = norm(sa);
aDir = sa/aLen;

bLen = norm(sb);
bDir = sb/bLen;

% from center to src and rec
toSrc = src-origin;
toRec = rec-origin;

srcLen = norm(toSrc);
toSrcDir = toSrc/srcLen;

recLen = norm(toRec);
toRecDir = toRec/recLen;

% convention: let planeNormal point to rec
toRecDot = toRecDir*planeNormal';
if toRecDot < 0
    planeNormal = -planeNormal;
    toRecDot = -toRecDot;
end

toSrcDot = toSrcDir*planeNormal';

if isAperture && toRecDot*toSrcDot > 0
    % we are on the same side of an aperture, goodbye
   return; 
end

if isImageSrc && toRecDot*toSrcDot > 0
    % the image source is on the same side of the reflector, goodbye
   return; 
end


% construct closest point on each side and distance
[edge(1).distance, edge(1).vecBetween, edge(1).pointOnEdge, tmp, edge(1).outOfBounds, tmp2, s] = DistBetween2Segments(verts(1,:), verts(2,:), src, rec);
[edge(2).distance, edge(2).vecBetween, edge(2).pointOnEdge, tmp, edge(2).outOfBounds, tmp2, s] = DistBetween2Segments(verts(2,:), verts(3,:), src, rec);
[edge(3).distance, edge(3).vecBetween, edge(3).pointOnEdge, tmp, edge(3).outOfBounds, tmp2, s] = DistBetween2Segments(verts(3,:), verts(4,:), src, rec);
[edge(4).distance, edge(4).vecBetween, edge(4).pointOnEdge, tmp, edge(4).outOfBounds, tmp2, s] = DistBetween2Segments(verts(4,:), verts(1,:), src, rec);

% note the perpendicular length for each edge
edge(1).perpenLen = bLen;
edge(2).perpenLen = aLen;
edge(3).perpenLen = bLen;
edge(4).perpenLen = aLen;

% this would be UTD but we also want the distance
% incident = excitant angle
% [pointOnEdge, s, outOfBounds] = diffractionPointOnEdge(verts(1,:), verts(2,:), src, rec);
          
% if any is out of bounds we must be past at least one shadow plane
% WARNING this all assumes that we have a rectangle to begin with!
isVisible = ~any([edge(:).outOfBounds]);

% what should the filter do:
% a) distance dependence: if src or receiver approach the plane any effect should dissappear 
% b) for diffraction and reflection: we want to fade out at or closely to
% the shadow boundary
% c) reflectionFilter = 1-diffractionFilter
% d) aperture: = reflectionFilter in the visible region but we still need
% diffraction in the shadow region
%c = 344;

%fRindel = c*astar/(2*S*cos(theta)) 

S = aLen*bLen;
astar = 2*recLen*srcLen/(recLen+srcLen); % Rindel distance

% for highpass in reflection/aperture, lowpass in diffraction
fRindel = c*astar/(2*S*-toSrcDot);  % this takes care of distance ->0 for one of the distances -> and -> inf for both distances -> inf
% in the 86 paper it is cos^2!!!!
% for 1x1m in 1m distance this is 172 Hz

% smallest distance to any edge, 0 on shadow plane
%minDist = min([edge(:).distance])/sqrt(S)*2; % normalized to size, 0 is on edge, for a square 1 is in the middle FIXME calculate per edge with respective diameter

[minDist, edgeNum] = min([edge(:).distance]); % normalized to size, 0 is on edge, for a square 1 is in the middle FIXME calculate per edge with respective diameter
% normalize to respective dimension
minDist = minDist/edge(edgeNum).perpenLen*2;


if ~isVisible
    minDist = -minDist; % negative if we are in the shadow region
end

% are we out of the diffraction area?
if minDist < -1
    return;
end

f3 = 100000000000;
keepHPcutoff = 0;
if keepHPcutoff 
    % lowering fBump and keeping the HP cutoff leads to widening of the bump
    % area which can be good to approximate the desired 0.5 order LP with the
    % one shelving filter
    fBump = fRindel*max(1+minDist,0)*2; % for shelving filter to get bump
    globalAtten = 1;
    
    if minDist > 0
        atten = max(1+minDist,0)*2/6;  % atten has to be limited for large distances
    else
        atten = (max(1+minDist,0)^4)*2/6;  % atten has to be limited for large distances
    end
    peak = 6/4;
    
    % modify fRindel so that we match the measurement in Rindel 2005 with the
    % combined HP and shelving filtet
    fRindel = fRindel * 1.7;
    
elseif 0
    % here we lower the HP cutoff in connection with an overall gain, to
    % lower the bump frequency
    
    globalAtten = max(1+minDist,0)/2; % cut 6 dB on edge
    fRindel = fRindel * globalAtten; % since we have a 6 dB per octave slope do the same with fc to keep the asymptotic behavior for fc -> 0
    
    % modify fRindel so that we match the measurement in Rindel 2005 with the
    % combined HP and shelving filtet
    fRindel = fRindel * 1.2;
    
   
    
    if minDist > 0
        atten = max(1+minDist/2,0)*4/1.5/6;  % atten has to be limited for large distances
    else
        atten = (max(1+minDist,0)^4)*4/1.5/6;  % atten has to be limited for large distances
    end
    peak = 6/4;
    
   % atten = 4/6;
    
 fBump = fRindel*(9 - atten*6); % for shelving filter to get bump, this is the freq where half atten is achieved for highShelvingMatched, not -3dB!
    
else 
    % combine with 3 dB filter in diffraction area
    
    % here we lower the HP cutoff in connection with an overall gain, to
    % lower the bump frequency
%     if minDist > 0
        globalAtten = max(1+minDist,0)/2; % cut 6 dB on edge, after this it is guesswork, just drops quite fast to nothing
%     else
%         globalAtten = sqrt(max(1+minDist,0))/2; % cut 6 dB on edge
%     end
    fRindel = fRindel * globalAtten; % since we have a 6 dB per octave slope do the same with fc to keep the asymptotic behavior for fc -> 0
    
    % modify fRindel so that we match the measurement in Rindel 2005 with the
    % combined HP and shelving filtet
    fRindel = fRindel * 1.2;
    
   
    
    if minDist >= 0
        atten = max(1+minDist/2,0)*4/1.5/6;  % atten has to be limited for large distances
        f3 = 100000000000;
    else
        % here minDist is negative
        atten = 4/1.5/6;  % atten has to be limited for large distances
     %   f3 = fRindel*10/(1-exp(minDist*10));
        f3 = fRindel*(10-1/minDist);
    end
    peak = 6/4;
    
   % atten = 4/6;
    
 fBump = fRindel*(9 - atten*6); % for shelving filter to get bump, this is the freq where half atten is achieved for highShelvingMatched, not -3dB!
    
  
    
end


% filter


% HP
%[b,a] = butter(1,fRindel*2/fs,'high');

% approximate analog HP with also cutoffs above fs/2 using the matched
% shelving filter design
f2 = fRindel/100250; % empirically fitted to match correctly for 100 Hz cutoff and 200 dB gain in shelving filter
% use shelving filter as HP with 200 dB gain re DC
[b,a] = highShelvingMatched(f2,200,fs);
b=b*10^(-200/20); % remove gain again to reach unity at f->inf


[b2,a2] = highShelvingMatched(fBump,20*log10(atten),fs);

% if ~keepHPcutoff
%     [b3,a3] = highShelvingMatched(sqrt(fBump*fs/2)/4,-20*log10(atten)/4,fs)
%     b2 = conv(b2, b3);
%     a2 = conv(a2, a3);
% end

b = conv(b, b2);
a = conv(a, a2);

% in diffraction area use -3dB/oct LP, otherwise we push f3 to inf
[b2,a2] = halfOrderLPapprox(f3,fs);
b = conv(b, b2);
a = conv(a, a2);

b= b*peak*globalAtten;

%[b2,a2] = butter(1,fBump*2/fs);

% % approximate analog HP with also cutoffs above fs/2 using the matched
% % shelving filter design
% fhp = 10000000;
% f2 = fhp/100250; % empirically fitted to match correctly for 100 Hz cutoff and 200 dB gain in shelving filter
% % use shelving filter as HP with 200 dB gain re DC
% [b,a] = highShelvingMatched(f2,200,fs)
% b=b*10^(-200/20); % remove gain again to reach unity at f->inf
% % compare to bilinear transf HP
% %[b,a] = butter(1,fhp*2/fs,'high');

% LP for rippled HP (delayed subtraction)
% fctest = 5000;
% fctest = fctest*2.5;
% [b,a] = butter(1,fctest*2/fs);



% % % approximate
% % % -3 dB/oct filter
% % ftarget = 100
% % 
% % % now in new function halfOrderLPapprox
% % % f3 = ftarget/4;  %ftarget cannot go below 1 Hz, gets unstable
% % % [b,a] = highShelvingMatched(f3,-3,fs);
% % % 
% % % [b2,a2] = highShelvingMatched(f3*10,-6,fs);
% % % b = conv(b, b2);
% % % a = conv(a, a2);
% % % 
% % % [b2,a2] = highShelvingMatched(f3*80,-9,fs);
% % % b = conv(b, b2);
% % % a = conv(a, a2);
% % % 
% % % [b2,a2] = highShelvingMatched(f3*1000,-12,fs);
% % % b = conv(b, b2);
% % % a = conv(a, a2);
% % % 
% % % % IRIfolp(f0,fs)
% % % a2 = exp(-(2*pi*f3*6000)/fs);
% % % b2 = 1 - a2;
% % % a2 = [1, -a2];
% % % b = conv(b, b2);
% % % a = conv(a, a2);
% % 
% % [b,a] = halfOrderLPapprox(ftarget,fs)
% % 
% % fvec = [10:20000];
% % 
% % % target
% % fOrder = 0.5;
% % blendExpn = 1.3;
% % target = 10*log10(1./(abs(fvec/ftarget).^(2*fOrder/blendExpn) + 1).^blendExpn);
% % figure(1)
% % semilogx(fvec,target)
% % hold on
% % % end of testing -3dB/oct filter approx

% %b2(1) = b2(1)/0.5+0,5;
% pulse =[1; zeros(fs-1,1)];
% out = filter(b,a,pulse);
% 
% % make ripple filter with delayed lp version
% % delay = round(fs/fctest/4);
% % out = pulse - [zeros(delay,1); out(1:end-delay)];
% 
% figure(1)
% semilogx(20*log10(abs(fft(out))))
% hold on
% xlim([10 20000])
% %ylim([-40 5])

