function recdir = transform_orientation(room, rel_recdir_deg, angle_flag)
% TRANSFORM_ORIENTATION - transform receiver orientation to overall coordinate system if specified
% as relative to src.
%
% Usage:
%   recdir = TRANSFORM_ORIENTATION(room, rel_recdir_deg, [angle_flag])
%
% Input:
%   room            room structure (see RAZR), must contain fields srcpos and recpos
%   rel_recdir_deg  Receiver orientation (deg), relative to source, as vector [azim, elev]
%   angle_flag      Specifies which angles are treated as being specified relative to the source:
%                   'azim': only azimuth, 'elev': only elevation, 'all': both of them.
%                   Optional parameter, default: 'all'. For any other input string none of the
%                   angles will be transformed.
%
% Output:
%   recdir          New values of the field room.recdir

%------------------------------------------------------------------------------
% RAZR engine for Mathwork's MATLAB
%
% Version 0.96.3
%
% Author(s): Torben Wendt
%
% Copyright (c) 2014-2023, Torben Wendt, Steven van de Par, Stephan Ewert,
% University of Oldenburg, Germany.
%
% This work is licensed under the
% Creative Commons Attribution-NonCommercial-NoDerivs 4.0 International
% License (CC BY-NC-ND 4.0).
% To view a copy of this license, visit
% http://creativecommons.org/licenses/by-nc-nd/4.0/ or send a letter to
% Creative Commons, 444 Castro Street, Suite 900, Mountain View, California,
% 94041, USA.
%------------------------------------------------------------------------------


if nargin < 3
    angle_flag = 'all';
end

if ~(isfield(room, 'srcpos') && isfield(room, 'recpos'))
    error('transform_orientation: room structure must contain fields srcpos and recpos.');
end

% vector pointing from rec to src
src_rec_conn = room.srcpos - room.recpos;

% % SE convert to azim and elev, depreciated but for now stay in this format
% [az, el] = cart2sph(src_rec_conn(1), src_rec_conn(2), src_rec_conn(3));
% 
% % SE DEBUG apply the desired azim and elev relative to the src
% [anglesOut] = transformAngles([az el]/pi*180,[rel_recdir_deg(1) rel_recdir_deg(2)],0,1); 
% angles = wrap_angles(anglesOut, 'deg');

% azim:
if strcmp(angle_flag, 'azim') || strcmp(angle_flag, 'all')
    % SE really? simplify
    recdir(1) = rel_recdir_deg(1) + rd2dg(cart2pol(src_rec_conn(1), src_rec_conn(2))); 
    %recdir(1) = angles(1);
else
    recdir(1) = rel_recdir_deg(1);
end

% elev:
if strcmp(angle_flag, 'elev') || strcmp(angle_flag, 'all')
    % SE FIXME this is wrong
    [az, el] = cart2sph(src_rec_conn(1), src_rec_conn(2), src_rec_conn(3));
    recdir(2) = rel_recdir_deg(2) + rd2dg(el);
    
    % SE disable for now
    %recdir(2) = angles(2);
    if rel_recdir_deg(2) ~= 0
        % SE FIXME
        error('Elevation for receiver not supported for transform_orientation');
    end
else
    recdir(2) = rel_recdir_deg(2);
end
